pub enum VmIoMemView<'a> {
WriteView(MemView),
ReadView(&'a MemView),
}Expand description
The memory view used for VM I/O operations.
The readers can think of this as a wrapped permission tokens for operating with a certain
memory view (see MemView) “owned” by the VmIoOwner that they are created from, which
are used for allowing callers to use VmReader and VmWriter to perform VM I/O operations.
For writers the memory permission must be exclusive so this enum contains an owned MemView
for the write view; for readers the memory permission can be shared so this enum contains a
reference to a MemView for the read view (which can be disabled optionally in VmSpaceOwner).
⚠️ WARNING: We do not recommend using this enum directly.
Variants
Implementations
Sourceimpl<'a> VmIoMemView<'a>
impl<'a> VmIoMemView<'a>
pub fn arrow_WriteView_0(self) -> MemView
pub fn arrow_ReadView_0(self) -> &'a MemView